@@ -12,6 +12,7 @@
 #include <Rmath.h>
 #include "locale.h"
 #include "dpq.h"
+#include "actuar.h"
 
 double dinvtrgamma(double x, double shape1, double shape2, double scale,
                    int give_log)
@@ -23,8 +24,10 @@ double dinvtrgamma(double x, double shape1, double shape2, double scale,
      *  with u = (scale/x)^shape2.
      */
 
-    double logu;
-
+#ifdef IEEE_754
+    if (ISNAN(x) || ISNAN(shape1) || ISNAN(shape2) || ISNAN(scale))
+	return x + shape1 + shape2 + scale;
+#endif
     if (!R_FINITE(shape1) ||
         !R_FINITE(shape2) ||
         !R_FINITE(scale)  ||
@@ -37,7 +40,7 @@ double dinvtrgamma(double x, double shape1, double shape2, double scale,
     if (!R_FINITE(x) || x <= 0.0)
         return ACT_D__0;
 
-    logu = shape2 * (log(scale) - log(x));
+    double logu = shape2 * (log(scale) - log(x));
 
     return ACT_D_exp(log(shape2) + shape1 * logu - exp(logu)
                    - log(x) - lgammafn(shape1));
@@ -46,8 +49,10 @@ double dinvtrgamma(double x, double shape1, double shape2, double scale,
 double pinvtrgamma(double q, double shape1, double shape2, double scale,
                    int lower_tail, int log_p)
 {
-    double u;
-
+#ifdef IEEE_754
+    if (ISNAN(q) || ISNAN(shape1) || ISNAN(shape2) || ISNAN(scale))
+	return q + shape1 + shape2 + scale;
+#endif
     if (!R_FINITE(shape1) ||
         !R_FINITE(shape2) ||
         !R_FINITE(scale)  ||
@@ -59,7 +64,7 @@ double pinvtrgamma(double q, double shape1, double shape2, double scale,
     if (q <= 0)
         return ACT_DT_0;
 
-    u = exp(shape2 * (log(scale) - log(q)));
+    double u = exp(shape2 * (log(scale) - log(q)));
 
     return pgamma(u, shape1, 1.0, !lower_tail, log_p);
 }
@@ -67,6 +72,10 @@ double pinvtrgamma(double q, double shape1, double shape2, double scale,
 double qinvtrgamma(double p, double shape1, double shape2, double scale,
                    int lower_tail, int log_p)
 {
+#ifdef IEEE_754
+    if (ISNAN(p) || ISNAN(shape1) || ISNAN(shape2) || ISNAN(scale))
+	return p + shape1 + shape2 + scale;
+#endif
     if (!R_FINITE(shape1) ||
         !R_FINITE(shape2) ||
         !R_FINITE(scale)  ||
@@ -98,6 +107,10 @@ double rinvtrgamma(double shape1, double shape2, double scale)
 double minvtrgamma(double order, double shape1, double shape2, double scale,
                    int give_log)
 {
+#ifdef IEEE_754
+    if (ISNAN(order) || ISNAN(shape1) || ISNAN(shape2) || ISNAN(scale))
+	return order + shape1 + shape2 + scale;
+#endif
     if (!R_FINITE(shape1) ||
         !R_FINITE(shape2) ||
         !R_FINITE(scale)  ||
@@ -117,8 +130,10 @@ double minvtrgamma(double order, double shape1, double shape2, double scale,
 double levinvtrgamma(double limit, double shape1, double shape2, double scale,
                      double order, int give_log)
 {
-    double u, tmp;
-
+#ifdef IEEE_754
+    if (ISNAN(limit) || ISNAN(shape1) || ISNAN(shape2) || ISNAN(scale) || ISNAN(order))
+	return limit + shape1 + shape2 + scale + order;
+#endif
     if (!R_FINITE(shape1) ||
         !R_FINITE(shape2) ||
         !R_FINITE(scale)  ||
@@ -128,17 +143,11 @@ double levinvtrgamma(double limit, double shape1, double shape2, double scale,
         scale  <= 0.0)
         return R_NaN;
 
-    if (order >= shape1 * shape2)
-	return R_PosInf;
-
     if (limit <= 0.0)
         return 0.0;
 
-    tmp = shape1 - order / shape2;
+    double u = exp(shape2 * (log(scale) - log(limit)));
 
-    u = exp(shape2 * (log(scale) - log(limit)));
-
-    return R_pow(scale, order) * gammafn(tmp)
-        * pgamma(u, tmp, 1.0, 0, 0) / gammafn(shape1)
+    return R_pow(scale, order) * gammaint_raw(u, shape1 - order/shape2) / gammafn(shape1)
         + ACT_DLIM__0(limit, order) * pgamma(u, shape1, 1.0, 1, 0);
 }
