@@ -11,6 +11,7 @@
 #include <Rmath.h>
 #include "locale.h"
 #include "dpq.h"
+#include "actuar.h"
 
 double dllogis(double x, double shape, double scale, int give_log)
 {
@@ -21,8 +22,10 @@ double dllogis(double x, double shape, double scale, int give_log)
      *  with u = v/(1 + v) = 1/(1 + 1/v), v = (x/scale)^shape.
      */
 
-    double tmp, logu, log1mu;
-
+#ifdef IEEE_754
+    if (ISNAN(x) || ISNAN(shape) || ISNAN(scale))
+	return x + shape + scale;
+#endif
     if (!R_FINITE(shape) ||
         !R_FINITE(scale) ||
         shape <= 0.0 ||
@@ -33,7 +36,7 @@ double dllogis(double x, double shape, double scale, int give_log)
         return ACT_D__0;
 
     /* handle x == 0 separately */
-    if (x == 0)
+    if (x == 0.0)
     {
 	if (shape < 1) return R_PosInf;
 	if (shape > 1) return ACT_D__0;
@@ -41,6 +44,8 @@ double dllogis(double x, double shape, double scale, int give_log)
 	return ACT_D_val(1.0 / scale);
     }
 
+    double tmp, logu, log1mu;
+
     tmp = shape * (log(x) - log(scale));
     logu = - log1pexp(-tmp);
     log1mu = - log1pexp(tmp);
@@ -50,8 +55,10 @@ double dllogis(double x, double shape, double scale, int give_log)
 
 double pllogis(double q, double shape, double scale, int lower_tail, int log_p)
 {
-    double u;
-
+#ifdef IEEE_754
+    if (ISNAN(q) || ISNAN(shape) || ISNAN(scale))
+	return q + shape + scale;
+#endif
     if (!R_FINITE(shape) ||
         !R_FINITE(scale) ||
         shape <= 0.0 ||
@@ -61,13 +68,17 @@ double pllogis(double q, double shape, double scale, int lower_tail, int log_p)
     if (q <= 0)
         return ACT_DT_0;
 
-    u = exp(-log1pexp(shape * (log(scale) - log(q))));
+    double u = exp(-log1pexp(shape * (log(scale) - log(q))));
 
     return ACT_DT_val(u);
 }
 
 double qllogis(double p, double shape, double scale, int lower_tail, int log_p)
 {
+#ifdef IEEE_754
+    if (ISNAN(p) || ISNAN(shape) || ISNAN(scale))
+	return p + shape + scale;
+#endif
     if (!R_FINITE(shape) ||
         !R_FINITE(scale) ||
         shape <= 0.0 ||
@@ -93,8 +104,10 @@ double rllogis(double shape, double scale)
 
 double mllogis(double order, double shape, double scale, int give_log)
 {
-    double tmp;
-
+#ifdef IEEE_754
+    if (ISNAN(order) || ISNAN(shape) || ISNAN(scale))
+	return order + shape + scale;
+#endif
     if (!R_FINITE(shape) ||
         !R_FINITE(scale) ||
         !R_FINITE(order) ||
@@ -106,7 +119,7 @@ double mllogis(double order, double shape, double scale, int give_log)
         order >= shape)
 	return R_PosInf;
 
-    tmp = order / shape;
+    double tmp = order / shape;
 
     return R_pow(scale, order) * gammafn(1.0 + tmp) * gammafn(1.0 - tmp);
 }
@@ -114,8 +127,10 @@ double mllogis(double order, double shape, double scale, int give_log)
 double levllogis(double limit, double shape, double scale, double order,
                  int give_log)
 {
-    double u, tmp1, tmp2, tmp3;
-
+#ifdef IEEE_754
+    if (ISNAN(limit) || ISNAN(shape) || ISNAN(scale) || ISNAN(order))
+	return limit + shape + scale + order;
+#endif
     if (!R_FINITE(shape) ||
         !R_FINITE(scale) ||
         !R_FINITE(order) ||
@@ -129,13 +144,10 @@ double levllogis(double limit, double shape, double scale, double order,
     if (limit <= 0.0)
         return 0;
 
-    tmp1 = order / shape;
-    tmp2 = 1.0 + tmp1;
-    tmp3 = 1.0 - tmp1;
+    double u = exp(-log1pexp(shape * (log(scale) - log(limit))));
+    double tmp = order / shape;
 
-    u = exp(-log1pexp(shape * (log(scale) - log(limit))));
-
-    return R_pow(scale, order) * gammafn(tmp2) * gammafn(tmp3)
-        * pbeta(u, tmp2, tmp3, 1, 0)
+    return R_pow(scale, order)
+	* betaint_raw(u, 1.0 + tmp, 1.0 - tmp)
         + ACT_DLIM__0(limit, order) * (0.5 - u + 0.5);
 }
